@@ -5,18 +5,18 @@ import time
 from shutil import copyfile
 
 from lutris import settings, sysoptions
-from lutris.runners import InvalidRunner, import_runner
+from lutris.runners import InvalidRunnerError, import_runner
 from lutris.util.log import logger
 from lutris.util.system import path_exists
 from lutris.util.yaml import read_yaml_from_file, write_yaml_to_file
 
 
-def make_game_config_id(game_slug):
+def make_game_config_id(game_slug: str) -> str:
     """Return an unique config id to avoid clashes between multiple games"""
     return "{}-{}".format(game_slug, int(time.time()))
 
 
-def write_game_config(game_slug, config):
+def write_game_config(game_slug: str, config: dict):
     """Writes a game config to disk"""
     configpath = make_game_config_id(game_slug)
     logger.debug("Writing game config to %s", configpath)
@@ -25,7 +25,7 @@ def write_game_config(game_slug, config):
     return configpath
 
 
-def duplicate_game_config(game_slug, source_config_id):
+def duplicate_game_config(game_slug: str, source_config_id: str):
     """Copies an existing configuration file, giving it a new id that this
     function returns."""
     new_config_id = make_game_config_id(game_slug)
@@ -36,7 +36,6 @@ def duplicate_game_config(game_slug, source_config_id):
 
 
 class LutrisConfig:
-
     """Class where all the configuration handling happens.
 
     Description
@@ -77,7 +76,7 @@ class LutrisConfig:
 
     """
 
-    def __init__(self, runner_slug=None, game_config_id=None, level=None):
+    def __init__(self, runner_slug: str = None, game_config_id: str = None, level: str = None):
         self.game_config_id = game_config_id
         if runner_slug:
             self.runner_slug = str(runner_slug)
@@ -122,7 +121,7 @@ class LutrisConfig:
     def runner_config_path(self):
         if not self.runner_slug:
             return None
-        return os.path.join(settings.CONFIG_DIR, "runners/%s.yml" % self.runner_slug)
+        return os.path.join(settings.RUNNERS_CONFIG_DIR, "%s.yml" % self.runner_slug)
 
     @property
     def game_config_path(self):
@@ -173,7 +172,7 @@ class LutrisConfig:
             self.merge_to_system_config(self.game_level.get("system"))
 
     def merge_to_system_config(self, config):
-        """Merge a configuration to the system configuation"""
+        """Merge a configuration to the system configuration"""
         if not config:
             return
         existing_env = None
@@ -204,8 +203,6 @@ class LutrisConfig:
 
     def remove(self):
         """Delete the configuration file from disk."""
-        if not self.game_config_path:
-            raise RuntimeError("Tried to remove a non-existent config")
         if not path_exists(self.game_config_path):
             logger.debug("No config file at %s", self.game_config_path)
             return
@@ -225,7 +222,8 @@ class LutrisConfig:
             config_path = self.game_config_path
         else:
             raise ValueError("Invalid config level '%s'" % self.level)
-
+        # Remove keys with no values from config before saving
+        config = {key: value for key, value in config.items() if value}
         logger.debug("Saving %s config to %s", self, config_path)
         write_yaml_to_file(config, config_path)
         self.initialize_config()
@@ -236,10 +234,17 @@ class LutrisConfig:
         defaults = {}
         for option, params in options_dict.items():
             if "default" in params:
-                defaults[option] = params["default"]
+                default = params["default"]
+                if callable(default):
+                    try:
+                        default = default()
+                    except Exception as ex:
+                        logger.exception("Unable to generate a default for '%s': %s", option, ex)
+                        continue
+                defaults[option] = default
         return defaults
 
-    def options_as_dict(self, options_type):
+    def options_as_dict(self, options_type: str) -> dict:
         """Convert the option list to a dict with option name as keys"""
         if options_type == "system":
             options = (
@@ -247,12 +252,12 @@ class LutrisConfig:
             )
         else:
             if not self.runner_slug:
-                return None
+                return {}
             attribute_name = options_type + "_options"
 
             try:
                 runner = import_runner(self.runner_slug)
-            except InvalidRunner:
+            except InvalidRunnerError:
                 options = {}
             else:
                 if not getattr(runner, attribute_name):
