@@ -1,6 +1,7 @@
 # Standard Library
 from gettext import gettext as _
 
+from lutris.exceptions import MissingGameExecutableError
 # Lutris Modules
 from lutris.runners.runner import Runner
 from lutris.util import system
@@ -10,6 +11,7 @@ class openmsx(Runner):
     human_name = _("openMSX")
     description = _("MSX computer emulator")
     platforms = [_("MSX, MSX2, MSX2+, MSX turboR")]
+    flatpak_id = "org.openmsx.openMSX"
     game_options = [
         {
             "option": "main_file",
@@ -22,5 +24,5 @@ class openmsx(Runner):
     def play(self):
         rom = self.game_config.get("main_file") or ""
         if not system.path_exists(rom):
-            return {"error": "FILE_NOT_FOUND", "file": rom}
-        return {"command": [self.get_executable(), rom]}
+            raise MissingGameExecutableError(filename=rom)
+        return {"command": self.get_command() + [rom]}
