@@ -4,19 +4,21 @@ import os
 import time
 from collections import defaultdict
 from gettext import gettext as _
+from typing import List
 from urllib.parse import parse_qsl, urlencode, urlparse
 
 from lxml import etree
 
 from lutris import settings
-from lutris.exceptions import AuthenticationError, UnavailableGame
+from lutris.exceptions import AuthenticationError, UnavailableGameError
 from lutris.installer import AUTO_ELF_EXE, AUTO_WIN32_EXE
 from lutris.installer.installer_file import InstallerFile
+from lutris.installer.installer_file_collection import InstallerFileCollection
 from lutris.services.base import OnlineService
 from lutris.services.service_game import ServiceGame
 from lutris.services.service_media import ServiceMedia
 from lutris.util import i18n, system
-from lutris.util.http import HTTPError, Request, UnauthorizedAccess
+from lutris.util.http import HTTPError, Request, UnauthorizedAccessError
 from lutris.util.log import logger
 from lutris.util.strings import human_size, slugify
 
@@ -27,6 +29,7 @@ class GogSmallBanner(ServiceMedia):
     size = (100, 60)
     dest_path = os.path.join(settings.CACHE_DIR, "gog/banners/small")
     file_pattern = "%s.jpg"
+    file_format = "jpeg"
     api_field = "image"
     url_pattern = "https:%s_prof_game_100x60.jpg"
 
@@ -46,7 +49,6 @@ class GogLargeBanner(GogSmallBanner):
 
 
 class GOGGame(ServiceGame):
-
     """Representation of a GOG game"""
     service = "gog"
 
@@ -88,11 +90,8 @@ class GOGService(OnlineService):
     token_path = os.path.join(settings.CACHE_DIR, ".gog.token")
     cache_path = os.path.join(settings.CACHE_DIR, "gog-library.json")
 
-    is_loading = False
-
     def __init__(self):
         super().__init__()
-        self.selected_extras = None
 
         gog_locales = {
             "en": "en-US",
@@ -125,25 +124,20 @@ class GOGService(OnlineService):
             return False
         try:
             user_data = self.get_user_data()
-        except UnauthorizedAccess:
+        except UnauthorizedAccessError:
             logger.warning("GOG token is invalid")
             return False
         return user_data and "username" in user_data
 
     def load(self):
         """Load the user game library from the GOG API"""
-        if self.is_loading:
-            logger.warning("GOG games are already loading")
-            return
         if not self.is_connected():
             logger.error("User not connected to GOG")
             return
-        self.is_loading = True
         games = [GOGGame.new_from_gog_game(game) for game in self.get_library()]
         for game in games:
             game.save()
         self.match_games()
-        self.is_loading = False
         return games
 
     def login_callback(self, url):
@@ -177,8 +171,9 @@ class GOGService(OnlineService):
         request = Request(url)
         try:
             request.get()
-        except HTTPError:
-            logger.error("Failed to get token, check your GOG credentials.")
+        except HTTPError as http_error:
+            logger.error(http_error)
+            logger.error("Failed to get token.")
             logger.warning("Clearing existing credentials")
             self.logout()
             return
@@ -231,10 +226,7 @@ class GOGService(OnlineService):
         try:
             request.get()
         except HTTPError:
-            logger.error(
-                "Failed to request %s, check your GOG credentials and internet connectivity",
-                url,
-            )
+            logger.error("Failed to request %s", url)
             return
         return request.json
 
@@ -294,21 +286,29 @@ class GOGService(OnlineService):
         return self.make_api_request(url)
 
     def get_download_info(self, downlink):
-        """Return file download information"""
+        """Return file download information, a list of dict containing the 'url' and
+        'filename' for each file."""
         logger.info("Getting download info for %s", downlink)
         try:
             response = self.make_api_request(downlink)
         except HTTPError as ex:
             logger.error("HTTP error: %s", ex)
-            raise UnavailableGame("The download of '%s' failed." % downlink) from ex
+            return []
         if not response:
-            raise UnavailableGame("The download of '%s' failed." % downlink)
+            logger.error("No download info obtained for %s", downlink)
+            return []
+
+        expanded = []
         for field in ("checksum", "downlink"):
             field_url = response[field]
             parsed = urlparse(field_url)
             query = dict(parse_qsl(parsed.query))
-            response[field + "_filename"] = os.path.basename(query.get("path") or parsed.path)
-        return response
+            filename = os.path.basename(query.get("path") or parsed.path)
+            expanded.append({
+                "url": response[field],
+                "filename": filename
+            })
+        return expanded
 
     def get_downloads(self, gogid):
         """Return all available downloads for a GOG ID"""
@@ -332,16 +332,19 @@ class GOGService(OnlineService):
         products = [game, *dlcs] if dlcs else [game]
         all_extras = {}
         for product in products:
-            extras = [
-                {
-                    "name": download.get("name", "").strip().capitalize(),
-                    "type": download.get("type", "").strip(),
-                    "total_size": download.get("total_size", 0),
-                    "id": str(download["id"]),
-                } for download in product["downloads"].get("bonus_content") or []
-            ]
-            if extras:
-                all_extras[product.get("title", "").strip()] = extras
+            # Extras for DLCs you don't own are listed, but are not installable.
+            if product.get("is_installable"):
+                extras = [
+                    {
+                        "name": download.get("name", "").strip().capitalize(),
+                        "type": download.get("type", "").strip(),
+                        "total_size": download.get("total_size", 0),
+                        "id": str(download["id"]),
+                        "downlinks": [f.get("downlink") for f in download.get("files") or []]
+                    } for download in product["downloads"].get("bonus_content") or []
+                ]
+                if extras:
+                    all_extras[product.get("title", "").strip()] = extras
         return all_extras
 
     def get_installers(self, downloads, runner, language="en"):
@@ -387,25 +390,31 @@ class GOGService(OnlineService):
             if not downlink:
                 logger.error("No download information for %s", game_file)
                 continue
-            download_info = self.get_download_info(downlink)
-            for field in ('checksum', 'downlink'):
+            for info in self.get_download_info(downlink):
                 download_links.append({
                     "name": download.get("name", ""),
                     "os": download.get("os", ""),
                     "type": download.get("type", ""),
                     "total_size": download.get("total_size", 0),
                     "id": str(game_file["id"]),
-                    "url": download_info[field],
-                    "filename": download_info[field + "_filename"]
+                    "url": info["url"],
+                    "filename": info["filename"]
                 })
         return download_links
 
-    def get_extra_files(self, downloads, installer):
+    def get_extra_files(self, installer, selected_extras):
         extra_files = []
-        for extra in downloads["bonus_content"]:
-            if str(extra["id"]) not in self.selected_extras:
-                continue
-            links = self.query_download_links(extra)
+        for extra in selected_extras:
+            if extra.get("downlinks"):
+                links = [info for link in extra.get("downlinks") for info in self.get_download_info(link)]
+            elif str(extra["id"]) in selected_extras:
+                links = self.query_download_links(extra)
+            else:
+                links = []
+
+            if not links:
+                logger.error("No download link for bonus content '%s' could be obtained.", extra.get("name"))
+
             for link in links:
                 if link["filename"].endswith(".xml"):
                     # GOG gives a link for checksum XML files for bonus content
@@ -430,7 +439,7 @@ class GOGService(OnlineService):
             _installer = gog_installers[0]
             return self.query_download_links(_installer)
         except HTTPError as err:
-            raise UnavailableGame("Couldn't load the download links for this game") from err
+            raise UnavailableGameError(_("Couldn't load the download links for this game")) from err
 
     def get_patch_files(self, installer, installer_file_id):
         logger.debug("Getting patches for %s", installer.version)
@@ -473,26 +482,32 @@ class GOGService(OnlineService):
             files.append(InstallerFile(installer.game_slug, file_id, {
                 "url": installer_file["url"],
                 "filename": installer_file["filename"],
-                "checksum_url": installer_file.get("checksum_url")
+                "checksum_url": installer_file.get("checksum_url"),
+                "total_size": installer_file["total_size"],
+                "size": -1,
             }))
         if not file_id_provided:
-            raise UnavailableGame("Unable to determine correct file to launch installer")
+            raise UnavailableGameError(_("Unable to determine correct file to launch installer"))
         return files
 
-    def get_installer_files(self, installer, installer_file_id):
+    def get_installer_files(self, installer, installer_file_id, selected_extras):
         try:
             downloads = self.get_downloads(installer.service_appid)
         except HTTPError as err:
-            raise UnavailableGame("Couldn't load the downloads for this game") from err
+            raise UnavailableGameError(_("Couldn't load the downloads for this game")) from err
         links = self._get_installer_links(installer, downloads)
         if links:
-            files = self._format_links(installer, installer_file_id, links)
+            files = [InstallerFileCollection(installer.game_slug, installer_file_id,
+                                             self._format_links(installer, installer_file_id, links))]
         else:
             files = []
-        if self.selected_extras:
-            for extra_file in self.get_extra_files(downloads, installer):
-                files.append(extra_file)
-        return files
+
+        extra_files = []
+        if selected_extras:
+            for extra_file in self.get_extra_files(installer, selected_extras):
+                extra_files.append(extra_file)
+
+        return files, extra_files
 
     def read_file_checksum(self, file_path):
         """Return the MD5 checksum for a GOG file
@@ -527,7 +542,7 @@ class GOGService(OnlineService):
             "name": db_game["name"],
             "version": "GOG",
             "slug": details["slug"],
-            "game_slug": slugify(db_game["name"]),
+            "game_slug": self.get_installed_slug(db_game),
             "runner": runner,
             "gogid": db_game["appid"],
             "script": {
@@ -540,6 +555,10 @@ class GOGService(OnlineService):
             }
         }
 
+    def get_installed_runner_name(self, db_game):
+        platforms = [platform.casefold() for platform in self.get_game_platforms(db_game)]
+        return "linux" if "linux" in platforms else "wine"
+
     def get_games_owned(self):
         """Return IDs of games owned by user"""
         url = "{}/user/data/games".format(self.embed_url)
@@ -562,7 +581,7 @@ class GOGService(OnlineService):
 
             for file in installfiles:
                 # supports linux
-                if file["os"].lower() == "linux":
+                if file["os"].casefold() == "linux":
                     runner = "linux"
                     script = [{"extract": {"dst": "$CACHE/GOG", "file": dlc_id, "format": "zip"}},
                               {"merge": {"dst": "$GAMEDIR", "src": "$CACHE/GOG/data/noarch/"}}]
@@ -576,7 +595,7 @@ class GOGService(OnlineService):
                     "version": f"{dlc['title']} ({runner})",
                     "slug": dlc["slug"],
                     "description": "DLC for %s" % db_game["name"],
-                    "game_slug": slugify(db_game["name"]),
+                    "game_slug": self.get_installed_slug(db_game),
                     "runner": runner,
                     "is_dlc": True,
                     "dlcid": dlc["id"],
@@ -625,7 +644,7 @@ class GOGService(OnlineService):
         patch_installers = []
         for version in patch_versions:
             patch = patch_versions[version]
-            size = human_size(sum([part["total_size"] for part in patch]))
+            size = human_size(sum(part["total_size"] for part in patch))
             patch_id = "gogpatch-%s" % slugify(patch[0]["version"])
             installer = {
                 "name": db_game["name"],
@@ -646,3 +665,11 @@ class GOGService(OnlineService):
             }
             patch_installers.append(installer)
         return patch_installers
+
+    def get_game_platforms(self, db_game: dict) -> List[str]:
+        details = db_game.get("details")
+        if details:
+            worksOn = json.loads(details).get("worksOn")
+            if worksOn is not None:
+                return [name for name, works in worksOn.items() if works]
+        return []
